/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */


#include "dps.h"



/*
 *  DPS States variable
 */


DPSstate __STATE = { // inicializacao dos estados
                     {0.0f, 0.0f, 0.0f},              // position
                     {0.0f, 0.0f, 0.0f},              // velocity
                     {0.0f, 0.0f, 0.0f, 1.0f},        // color
                     {0.0f, 0.0f, 0.0f, 0.0f},        // color dev
                     0.0f,                            // age
                     0.0f,                            // mass
                     DPS_FALSE,                       // roll
                     DPS_NULL,                        // mesh
                     DPS_NULL,                        // collide
                     DPS_NULL                         // die
                   };



/*
 *  Particle Groups Global State Machine
 */

GList*      __DPS_GROUP_PARTICLE_SYSTEM[DPS_MAX_PARTICLE_GROUPS];
DPSulong    __DPS_COUNTER_PARTICLE_SYSTEM = 0;

GList*      __DPS_CURRENT_PARTICLE_SYSTEM;
DPSulong    __DPS_CURRENT_PARTICLE_SYSTEM_INDEX;


/*
 *  Force Groups Global State Machine
 */

GList*      __DPS_GROUP_FORCE_SYSTEM[DPS_MAX_FORCE_GROUPS];
DPSulong    __DPS_COUNTER_FORCE_SYSTEM = 0;

GList*      __DPS_CURRENT_FORCE_SYSTEM;
DPSulong    __DPS_CURRENT_FORCE_SYSTEM_INDEX;


/*
 *  Object Groups Global State Machine
 */

GList*      __DPS_GROUP_OBJECT_SYSTEM[DPS_MAX_OBJECT_GROUPS];
DPSulong    __DPS_COUNTER_OBJECT_SYSTEM = 0;

GList*      __DPS_CURRENT_OBJECT_SYSTEM;
DPSulong    __DPS_CURRENT_OBJECT_SYSTEM_INDEX;







/*
 *  State functions
 */

void dpsTimeIncrementf(DPSfloat dt) {  // define o dt;
  __STATE.__DPS_STATE_DT = dt;
}


void dpsColor4f(DPSfloat r, DPSfloat g, DPSfloat b, DPSfloat a) {
  __STATE.__DPS_STATE_COLOR[0] = r;
  __STATE.__DPS_STATE_COLOR[1] = g;
  __STATE.__DPS_STATE_COLOR[2] = b;
  __STATE.__DPS_STATE_COLOR[3] = a;
}




void dpsColor3f(DPSfloat r, DPSfloat g, DPSfloat b) {
  dpsColor4f(r, g, b, 1.0f);
}




void dpsRGBADev(DPSfloat rdev, DPSfloat gdev, DPSfloat bdev, DPSfloat adev) {
  __STATE.__DPS_STATE_COLOR_DEV[0]    = rdev;
  __STATE.__DPS_STATE_COLOR_DEV[1]    = gdev;
  __STATE.__DPS_STATE_COLOR_DEV[2]    = bdev;
  __STATE.__DPS_STATE_COLOR_DEV[3]    = adev;
}




void dpsPosition3f(DPSfloat rx, DPSfloat ry, DPSfloat rz) {
  __STATE.__DPS_STATE_POSITION[0] = rx;
  __STATE.__DPS_STATE_POSITION[1] = ry;
  __STATE.__DPS_STATE_POSITION[2] = rz;
}




void dpsVelocity3f(DPSfloat vx, DPSfloat vy, DPSfloat vz) {
  __STATE.__DPS_STATE_VELOCITY[0] = vx;
  __STATE.__DPS_STATE_VELOCITY[1] = vy;
  __STATE.__DPS_STATE_VELOCITY[2] = vz;
}


void dpsMeshFunc(DPSvoid (*mesh)(DPSulong)) {
  __STATE.__DPS_STATE_MESH = mesh;
}

void dpsMeshRoll(DPSboolean roll) {
  __STATE.__DPS_STATE_ROLL = roll;
}



void dpsCollideFunc(DPSvoid (*collide)(DPShandle, DPSshandle, 
                      DPSshandle, DPSshandle)) {
  __STATE.__DPS_STATE_COLLIDE = collide;
}



void dpsDieFunc(DPSvoid (*die)(DPShandle, DPSshandle, 
                      DPSshandle, DPSshandle)) {
  __STATE.__DPS_STATE_DIE = die;
}


void dpsAgef(DPSfloat age) {
  __STATE.__DPS_STATE_AGE = age;
}

void dpsMassf(DPSfloat mass) {
  __STATE.__DPS_STATE_MASS = mass;
}






/*
 **************************************************************************
 *                        CUT HERE (MIGUEL)                               *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Particle Systems 
 */



DPSshandle dpsGenParticleSystem(DPSsizei range) {
  DPSsizei i;
  dpsLoadTables();   // Inicializa funcoes mapeadas. Soh carrega uma vez
  for(i=0; i<range-1; i++)
    __DPS_GROUP_PARTICLE_SYSTEM[__DPS_COUNTER_PARTICLE_SYSTEM+i] = DPS_NULL;
  return __DPS_COUNTER_PARTICLE_SYSTEM+=range;
}


void dpsNewParticleSystem(DPSshandle handle) {
  __DPS_CURRENT_PARTICLE_SYSTEM_INDEX = handle;
  __DPS_CURRENT_PARTICLE_SYSTEM       = __DPS_GROUP_PARTICLE_SYSTEM[handle];
}


void dpsEndParticleSystem() {
  __DPS_GROUP_PARTICLE_SYSTEM[__DPS_CURRENT_PARTICLE_SYSTEM_INDEX] = 
    __DPS_CURRENT_PARTICLE_SYSTEM;
}



DPShandle dpsNewParticle() {
  DPShandle h;
  __DPS_CURRENT_PARTICLE_SYSTEM = 
      g_list_append(__DPS_CURRENT_PARTICLE_SYSTEM, 
         h = dpsNewParticle10v(__STATE.__DPS_STATE_POSITION,
                               __STATE.__DPS_STATE_VELOCITY,
                               __STATE.__DPS_STATE_MASS,
                               __STATE.__DPS_STATE_AGE,
                               __STATE.__DPS_STATE_COLOR,
                               __STATE.__DPS_STATE_COLOR_DEV,
                               __STATE.__DPS_STATE_ROLL,
                               __STATE.__DPS_STATE_MESH,
                               __STATE.__DPS_STATE_DIE,
                               __STATE.__DPS_STATE_COLLIDE));
  return h;                            
}






DPSulong dpsLiveParticles(DPSshandle handle) {
  return g_list_length(__DPS_GROUP_PARTICLE_SYSTEM[handle]);
}


void dpsParticleColor4fv(DPSvector4f color, DPSparticle *p) {
  color[0] = p->color[0];
  color[1] = p->color[1];
  color[2] = p->color[2];
  color[3] = p->color[3];
}


void dpsParticlePosition3fv(DPSvector3f position, DPSparticle *p) {
  position[0] = p->position[0];
  position[1] = p->position[1];
  position[2] = p->position[2];
}


void dpsParticleVelocity3fv(DPSvector3f velocity, DPSparticle *p) {
  velocity[0] = p->velocity[0];
  velocity[1] = p->velocity[1];
  velocity[2] = p->velocity[2];
}


void dpsRemoveParticle(DPSshandle shandle, DPShandle phandle) {
  __DPS_GROUP_PARTICLE_SYSTEM[shandle]=
            g_list_remove(__DPS_GROUP_PARTICLE_SYSTEM[shandle], phandle);
}

/*
 * Esta funcao pode ser altamente optimizada.
 * tanto ao nivel da deteccao da maskara e do modo de desenho, 
 * assim como na forma como se manda o gl desenhar ....
 */
void dpsDrawParticleSystem(DPSbitfield mask, DPSshandle handle) {
  GList *l; DPSvector3f tmp, tmp2;


  if(DPS_POINTS & mask) {  // draw points
    glBegin(GL_POINTS);
      for(l = __DPS_GROUP_PARTICLE_SYSTEM[handle]; l; l=g_list_next(l)) {
        if(DPS_COLOR & mask) glColor4fv(((DPSparticle*)l->data)->color); // set color
        glVertex3fv(((DPSparticle*)l->data)->position);
      }
    glEnd();
  } else if(DPS_LINES & mask) { // draw lines
     for(l = __DPS_GROUP_PARTICLE_SYSTEM[handle]; l; l=g_list_next(l)) {
     
       if(DPS_COLOR & mask) glColor4fv(((DPSparticle*)l->data)->color); // set color
       // se a velocidade == 0 desenha pontos em vez de linhas
       if(dpsSqrLengthVect(tmp, ((DPSparticle*)l->data)->velocity) < DPS_EPSILON) {
         glBegin(GL_POINTS);
	   glVertex3fv(((DPSparticle*)l->data)->position);
         glEnd();
       } else {
         glBegin(GL_LINES);
	   glVertex3fv(((DPSparticle*)l->data)->position);
	   glVertex3fv(dpsSubVect(tmp, ((DPSparticle*)l->data)->position, ((DPSparticle*)l->data)->velocity));
         glEnd();
       }
     }
  } else if(DPS_MESHES & mask) { // draw meshes
     for(l = __DPS_GROUP_PARTICLE_SYSTEM[handle]; l; l=g_list_next(l)) {
       glPushMatrix();

       glTranslatef(((DPSparticle*)l->data)->position[0],
                    ((DPSparticle*)l->data)->position[1],
                    ((DPSparticle*)l->data)->position[2]);

       if(((DPSparticle*)l->data)->roll) {  // rodar mesh no sentido da vel.
         tmp[0] = tmp[2] = 0.0f; tmp[1] = 1.0f;
	 dpsProdVect(tmp2, tmp, ((DPSparticle*)l->data)->velocity);
         glRotatef(dpsAngle2v(tmp, ((DPSparticle*)l->data)->velocity), tmp2[0], tmp2[1], tmp2[2]);
       }
       ((DPSparticle*)l->data)->mesh(((DPSparticle*)l->data)->mesh_arg++);  // exec draw mesh func
       glPopMatrix();
     }
  }
}








void dpsIterateParticles(DPSshandle particles, 
		         DPSshandle forces, 
			 DPSshandle objects) {

     DPSvector3f tmp;
     DPSsets sets = {particles, forces, objects};
     GList *f;
     GList *o;
     GList *p;
     
     for(f = __DPS_GROUP_FORCE_SYSTEM[forces]; f; f=g_list_next(f)) {
       // aqui eh preciso passar os shandles para serem executados
       // as funcoes de collide e die
       dpsApplyForce(f->data, 
                     &__DPS_GROUP_PARTICLE_SYSTEM[particles],
                     &__STATE, 
		     sets);
     }



     for(o = __DPS_GROUP_OBJECT_SYSTEM[objects]; o; o=g_list_next(o)) {
       // aqui eh preciso passar os shandles para serem executados
       // as funcoes de collide e die
       dpsApplyObject(o->data, 
                      &__DPS_GROUP_PARTICLE_SYSTEM[particles],
                      &__STATE, 
		      sets);
       
     }

       
     // aplicar o vector resultante a' posicao da particula         
     for(p = __DPS_GROUP_PARTICLE_SYSTEM[particles]; p; p=g_list_next(p)) {
       dpsMulVectConst(tmp, ((DPSparticle*)(p->data))->velocity, __STATE.__DPS_STATE_DT);
       dpsAddVect(((DPSparticle*)(p->data))->position, ((DPSparticle*)(p->data))->position, tmp);
       ((DPSparticle*)(p->data))->age+=__STATE.__DPS_STATE_DT; // envelhece a particula
     }
}
/*
 **************************************************************************
 *                        CUT HERE END (MIGUEL)                           *
 **************************************************************************
 */













/*
 **************************************************************************
 *                        CUT HERE (GUERRA)                               *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Force Systems 
 */

void dpsRemoveForce(DPSshandle shandle, DPShandle fhandle) {
  __DPS_GROUP_FORCE_SYSTEM[shandle]=
           g_list_remove(__DPS_GROUP_FORCE_SYSTEM[shandle], fhandle);
}



DPSshandle dpsGenForceSystem(DPSsizei range) {
  DPSsizei i;
  dpsLoadTables();   // Inicializa funcoes mapeadas. Soh carrega uma vez
  for(i=0; i<range-1; i++)
    __DPS_GROUP_FORCE_SYSTEM[__DPS_COUNTER_FORCE_SYSTEM+i] = DPS_NULL;
  return __DPS_COUNTER_FORCE_SYSTEM+=range;
}


void dpsNewForceSystem(DPSshandle handle) {
  __DPS_CURRENT_FORCE_SYSTEM_INDEX = handle;
  __DPS_CURRENT_FORCE_SYSTEM       = __DPS_GROUP_FORCE_SYSTEM[handle];
}





void dpsEndForceSystem() {
  __DPS_GROUP_FORCE_SYSTEM[__DPS_CURRENT_FORCE_SYSTEM_INDEX] = 
    __DPS_CURRENT_FORCE_SYSTEM;
}








DPShandle dpsNewExplosion(DPSfloat x, DPSfloat y, DPSfloat z,
                          DPSfloat velocity, 
			  DPSfloat magnitude, 
			  DPSfloat stdev, 
			  DPSfloat epsilon,
			  DPSfloat age) {
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
        h = dpsNewExplosion8f(x, y, z, velocity, magnitude, stdev, epsilon, age));
  return h;	
}






DPShandle dpsNewJet(DPSfloat cx, DPSfloat cy, DPSfloat cz,
	       DPSfloat ax, DPSfloat ay, DPSfloat az,		     
	       DPSfloat magnitude, 
	       DPSfloat epsilon,
	       DPSfloat max_radius){
  DPShandle h;
/*     __DPS_CURRENT_FORCE_SYSTEM =  */
/* 	g_list_append(__DPS_CURRENT_FORCE_SYSTEM, */
/*          dpsNewJet9f(cx, cy, cz, ax, ay, az, magnitude, epsilon, max_radius)); */
  return h;
}


DPShandle dpsNewGravity(DPSfloat x, DPSfloat y, DPSfloat z) {
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
         h = dpsNewGravity3f(x, y, z));
  return h;	 
}




DPShandle dpsNewFollow(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius) {
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
        h = dpsNewFollow3f(magnitude, epsilon, max_radius));
  return h;	
}



DPShandle dpsNewGravitate(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius) {
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
        h=  dpsNewGravitate3f(magnitude, epsilon, max_radius));
  return h;	
}

DPShandle dpsNewSpeedLimit(DPSfloat min, DPSfloat max){
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
         h = dpsNewSpeedLimit2f(min, max));
  return h;	 
}



DPShandle dpsNewTargetColor(DPSfloat c1, DPSfloat c2, DPSfloat c3, DPSfloat alpha, DPSfloat scale){
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
	h = dpsNewTargetColor5f(c1, c2, c3, alpha, scale));
  return h;	
}



DPShandle dpsNewTargetVelocity(DPSfloat v1, DPSfloat v2, DPSfloat v3, DPSfloat scale){
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
	h = dpsNewTargetVelocity4f(v1, v2, v3, scale));
  return h;	
}



DPShandle dpsNewKillOld(DPSfloat age_limit, DPSboolean  kill_less_than){
  DPShandle h;
  __DPS_CURRENT_FORCE_SYSTEM = 
    g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
		h =  dpsNewKillOld2f(age_limit, kill_less_than));
  return h;		
}






DPShandle dpsNewDamping(DPSfloat d0, DPSfloat d1, DPSfloat d2, DPSfloat vlowSqr, DPSfloat vhighSqr){
  DPShandle h;
    __DPS_CURRENT_FORCE_SYSTEM = 
	g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
		h = dpsNewdamping5f(d0, d1, d2, vlowSqr, vhighSqr));
  return h;		
}



DPShandle psNewOrbitLine(DPSfloat p0, 
		  DPSfloat p1,  
		  DPSfloat p2, 
		  DPSfloat axis0, 
		  DPSfloat axis1,
		  DPSfloat axis2, 
		  DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){

DPShandle h;
    __DPS_CURRENT_FORCE_SYSTEM = 
	g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
	  h = dpsNewOrbitLine9f(p0, p1, p2, axis0, axis1, axis2, magnitude, epsilon, max_radius));
}





void dpsNewOrbitPoint(DPSfloat c0, DPSfloat c1, DPSfloat c2, 
			DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){
/*     __DPS_CURRENT_FORCE_SYSTEM =   */
/* 	g_list_append(__DPS_CURRENT_FORCE_SYSTEM,  */
/* 		      dpsNewOrbitPoint6f(c0, c1, c2, magnitude, epsilon, max_radius)); */ 
}                 




DPShandle dpsNewVortex(DPSfloat p0, 
		  DPSfloat p1,  
		  DPSfloat p2, 
		  DPSfloat axis0, 
		  DPSfloat axis1,
		  DPSfloat axis2, 
		  DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){
  DPShandle h;
    __DPS_CURRENT_FORCE_SYSTEM = 
	g_list_append(__DPS_CURRENT_FORCE_SYSTEM,
          h = dpsNewVortex9f(p0, p1, p2, axis0, axis1, axis2, magnitude, 
	        epsilon, max_radius));
  return h;		
}




/*
 **************************************************************************
 *                        CUT HERE END (GUERRA)                           *
 **************************************************************************
 */













/*
 **************************************************************************
 *                        CUT HERE (GABI)                                 *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Object Systems 
 */


void dpsRemoveObject(DPSshandle shandle, DPShandle ohandle) {
  __DPS_GROUP_OBJECT_SYSTEM[shandle]=
           g_list_remove(__DPS_GROUP_OBJECT_SYSTEM[shandle], ohandle);
}


DPSshandle dpsGenObjectSystem(DPSsizei range) {
  DPSsizei i;
  dpsLoadTables();   // Inicializa funcoes mapeadas. Soh carrega uma vez
  for(i=0; i<range-1; i++)
    __DPS_GROUP_OBJECT_SYSTEM[__DPS_COUNTER_OBJECT_SYSTEM+i] = DPS_NULL;
  return __DPS_COUNTER_OBJECT_SYSTEM+=range;
}


void dpsNewObjectSystem(DPSshandle handle) {
  __DPS_CURRENT_OBJECT_SYSTEM_INDEX = handle;
  __DPS_CURRENT_OBJECT_SYSTEM       = __DPS_GROUP_OBJECT_SYSTEM[handle];
}


void dpsEndObjectSystem() {
  __DPS_GROUP_OBJECT_SYSTEM[__DPS_CURRENT_OBJECT_SYSTEM_INDEX] = 
    __DPS_CURRENT_OBJECT_SYSTEM;
}


/*
 * Declaracao de novos objectos
 *
 */


DPShandle dpsNewSphere(DPSfloat x, DPSfloat y, DPSfloat z,
		  DPSfloat ri, DPSfloat ro, 
		  DPSenum action, DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewSphere9f(x, y, z, ri, ro, action, var1, var2, var3));
  return h;           
}



DPShandle dpsNewPlane(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		      DPSfloat x1, DPSfloat y1, DPSfloat z1,
		      DPSenum action, DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewPlane10f(x0, y0, z0, x1, y1,z1, action, var1, var2, var3));
  return h;
}


DPShandle dpsNewRectangle(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat x2, DPSfloat y2, DPSfloat z2,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3) {

  DPShandle h;
  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
         h = dpsNewRectangle13f(x0, y0, z0, x1, y1, z1, x2, y2, z2, 
	                      action, var1, var2, var3));
  return h;
}



DPShandle dpsNewTriangle(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat x2, DPSfloat y2, DPSfloat z2,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewTriangle13f(x0, y0, z0, x1, y1, z1, x2, y2, z2, 
	                      action, var1, var2, var3));
  return h;
}





DPShandle dpsNewDisc(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		   DPSfloat x1, DPSfloat y1, DPSfloat z1,
		   DPSfloat ri, DPSfloat ro, DPSenum action, 
		   DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewDisc12f(x0, y0, z0, x1, y1,z1,ri,ro, action, var1, var2, var3));
  return h;
}

DPShandle dpsNewCylinder(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		         DPSfloat x1, DPSfloat y1, DPSfloat z1,
		         DPSfloat ri, DPSfloat ro, DPSenum action, 
		         DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewCylinder12f(x0, y0, z0, x1, y1,z1,ri,ro, action, var1, var2, var3));
  return h;
}

DPShandle dpsNewCone(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		   DPSfloat x1, DPSfloat y1, DPSfloat z1,
		   DPSfloat ri, DPSfloat ro, DPSenum action, 
		   DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewCone12f(x0, y0, z0, x1, y1,z1,ri,ro, action, var1, var2, var3));
  return h;
}





DPShandle dpsNewCube(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSenum action,
		     DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  DPShandle h;

  __DPS_CURRENT_OBJECT_SYSTEM = 
    g_list_append(__DPS_CURRENT_OBJECT_SYSTEM,
           h = dpsNewCube10f(x0, y0, z0, x1, y1,z1, action, var1, var2, var3));
  return h;
}






/*
 **************************************************************************
 *                        CUT HERE END (GABI)                             *
 **************************************************************************
 */


