/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "particle.h"



// alloc's memory for a particle
DPSparticle* dpsNewParticleAlloc() {
  return (DPSparticle*)malloc(sizeof(DPSparticle));
}




// alloc's memory for a particle and set's attributes
inline DPSparticle* dpsNewParticleAttrAll(
  DPSfloat rx, DPSfloat ry, DPSfloat rz,
  DPSfloat vx, DPSfloat vy, DPSfloat vz, 
  DPSfloat mass, DPSfloat age,
  DPSfloat red, DPSfloat green, DPSfloat blue, DPSfloat alpha,
  DPSfloat reddev, DPSfloat greendev, DPSfloat bluedev, DPSfloat alphadev,
  DPSboolean roll,  
  DPSvoid (*mesh)(DPSulong),
  DPSvoid (*die)(DPShandle, DPSshandle, DPSshandle, DPSshandle),
  DPSvoid (*collide)(DPShandle, DPSshandle, DPSshandle, DPSshandle)) { 

  DPSparticle *p = dpsNewParticleAlloc();

  p->position[0] = rx;
  p->position[1] = ry;
  p->position[2] = rz;

  p->velocity[0] = vx;
  p->velocity[1] = vy;
  p->velocity[2] = vz;
   
  p->color[0]    = dpsRandomNormalDist(red, reddev);
  p->color[1]    = dpsRandomNormalDist(green, greendev);
  p->color[2]    = dpsRandomNormalDist(blue, bluedev);
  p->color[3]    = dpsRandomNormalDist(alpha, alphadev);

  p->roll        = roll;
  p->mesh        = mesh;
  p->mesh_arg    = 0;
  p->mass        = mass;
  p->age         = age;
  p->die         = die;
  p->collide     = collide;
  return p;
}



DPSparticle* dpsNewParticle10v(
  DPSvector3f position, 
  DPSvector3f velocity,
  DPSfloat mass, 
  DPSfloat age, 
  DPSvector4f color,
  DPSvector4f colordev,
  DPSboolean roll,  
  DPSvoid (*mesh)(DPSulong), 
  DPSvoid (*die)(DPShandle, DPSshandle, DPSshandle, DPSshandle),
  DPSvoid (*collide)(DPShandle, DPSshandle, DPSshandle, DPSshandle)
) {
                                
  return 
    dpsNewParticleAttrAll(
      position[0], position[1], position[2],
      velocity[0], velocity[1], velocity[2],
      mass, age,
      color[0], 
      color[1], 
      color[2], 
      color[3], 
      colordev[0], 
      colordev[1], 
      colordev[2], 
      colordev[3], 
      roll, mesh, die, collide);
}
